import os
# import sys

try:
    from .ui_nodesummary import Ui_NodeSummary
except ImportError:
    dir = os.path.dirname(__file__)
    # sys.path.append(dir)
    target = os.path.join(dir, 'ui_nodesummary.py')
    if not os.path.exists(target):
        os.system(
            f'cd {dir} && pyside6-uic nodesummary.ui -o ui_nodesummary.py')

from .ui_nodesummary import Ui_NodeSummary
from .nodesummary import NodeSummary



from typing import Any, Optional, Union, Dict
import PySide6.QtGui
from PySide6.QtWidgets import QLabel, QGraphicsItem, QGraphicsWidget, QGraphicsProxyWidget, QGraphicsSceneHoverEvent, QGraphicsSceneMouseEvent, QStyleOptionGraphicsItem, QWidget, QGraphicsLinearLayout, QGraphicsTextItem, QGraphicsGridLayout
from PySide6.QtCore import Signal, Qt, QRectF, QLineF, QPointF
from PySide6.QtGui import QPainter, QColor, QPen, QFont, QFontMetrics, QPalette, QBrush
from ...ir import Variable, Node
import math
import abc


class GraphNode(QGraphicsWidget):
    _id_counter: int = 0

    pos_move = Signal(list)
    io_change = Signal(list)

    def __init__(self, parent=None):
        super().__init__()

        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, True)
        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, True)
        self.setAcceptHoverEvents(True)
        self.setZValue(2)

        self._hovered = False

        self._id = GraphNode._id_counter
        GraphNode._id_counter += 1

    @property
    def id(self):
        return self._id

    def layoutWith(self, op_type: str, name: Union[str, None], attrs: Union[None, Dict[str, Any]]):
        layout = QGraphicsLinearLayout(Qt.Orientation.Vertical, self)

        def gen_txt(txt, size, weight, color=QColor("black")):
            label = QLabel(txt)
            label.setFont(QFont('Monospace', size, weight))
            palette = label.palette()
            palette.setColor(QPalette.ColorRole.WindowText, color)
            palette.setColor(QPalette.ColorRole.Window,
                             Qt.GlobalColor.transparent)
            label.setPalette(palette)
            proxy = QGraphicsProxyWidget()
            proxy.setWidget(label)
            return proxy

        op_type_size = 14
        op_type_weight = QFont.Weight.Bold
        if name is not None:
            layout.addItem(
                gen_txt(name, 14, QFont.Weight.Bold), QColor(255, 255, 255))
            op_type_weight = QFont.Weight.Medium
            op_type_size = 13

        assert op_type is not None
        layout.addItem(gen_txt(op_type, op_type_size,
                       op_type_weight, QColor(255, 255, 255)))

        if attrs is not None:
            if len(attrs) > 0:
                attrs_layout = QGraphicsGridLayout()
                for i, (k, v) in enumerate(attrs.items()):
                    key_item = gen_txt(
                        k, 10, QFont.Weight.Normal, QColor(255, 255, 255))
                    attrs_layout.addItem(key_item, i, 0)

                    if isinstance(v, Variable):
                        v = f'{v.type.value}<{v.shape}>'
                    value_item = gen_txt(
                        str(v), 10, QFont.Weight.Thin, QColor(255, 255, 255))
                    attrs_layout.addItem(value_item, i, 1)
                layout.addItem(attrs_layout)

        self.setLayout(layout)
        self.adjustSize()

    def hoverEnterEvent(self, event: QGraphicsSceneHoverEvent) -> None:
        self._hovered = True
        self.update()
        return super().hoverEnterEvent(event)

    def hoverLeaveEvent(self, event: QGraphicsSceneHoverEvent) -> None:
        self._hovered = False
        self.update()
        return super().hoverLeaveEvent(event)

    def mouseDoubleClickEvent(self, event: QGraphicsSceneMouseEvent) -> None:
        return super().mouseDoubleClickEvent(event)

    def mouseMoveEvent(self, event: QGraphicsSceneMouseEvent) -> None:
        super().mouseMoveEvent(event)

    def itemChange(self, change: QGraphicsItem.GraphicsItemChange, value: Any) -> Any:
        if change == QGraphicsItem.GraphicsItemChange.ItemPositionHasChanged:
            self.pos_move.emit([self])
        return super().itemChange(change, value)

    def paint(self, painter: QPainter, option, widget=None):
        color = QColor(255, 165, 0) if self.isSelected(
        ) else QColor(255, 255, 255)
        width = 1.5 if self._hovered else 1
        painter.setPen(QPen(color, width))
        painter.setBrush(QBrush(QColor(100, 100, 100)))
        painter.drawRoundedRect(self.boundingRect(), 10, 10)

    def connectToEdge(self): ...



from typing import Dict, Any, Set, List
from types import FunctionType
import json


class TypeCheckMeta(type):
    def __setattr__(cls, name, value):
        expected_type = cls.__annotations__.get(name)
        print(f'{name} -> {expected_type}')
        if expected_type is not None and not isinstance(value, expected_type):
            raise TypeError(
                f"Expected type {expected_type} for attribute {name}, but got type {type(value)}")
        super().__setattr__(name, value)


class IRObj(metaclass=TypeCheckMeta):
    _id_map: List['IRObj'] = []

    def __init__(self) -> None:
        self._ext: Dict[str, Any] = {
            'display_attr': {},
            'strict_type': {},
        }
        self._id = len(self._id_map)
        self._id_map.append(self)

    @property
    def id(self):
        return self._id

    @staticmethod
    def getById(id):
        return IRObj._id_map[id]

    def read_ext(self, key):
        assert key in self._ext, key
        return self._ext[key]

    def set_ext(self, key, v):
        assert key in self._ext, key
        self._ext[key] = v

    def displayAttr(self, attr: str, name=None):
        if name is None:
            name = attr
        assert name not in self._ext['display_attr']
        self._ext['display_attr'][name] = attr

    def markStrictType(self, attr, type):
        assert attr not in self._ext['strict_type']
        self._ext['strict_type'][attr] = type

    def toJson(self) -> dict:
        def walk(v):
            if isinstance(v, IRObj):
                return v.toJson()
            elif isinstance(v, (set, list, tuple)):
                return type(v)([walk(vv) for vv in v])
            elif isinstance(v, dict):
                return {kk: walk(vv) for kk, vv in v.items()}
            else:
                return v
        return {k: walk(getattr(self, v)) for k, v in self._ext['display_attr'].items()}

    def __setattr__(self, name, value):
        if name != '_ext':
            if name in self._ext['strict_type']:
                if not isinstance(value, self._ext['strict_type'][name]):
                    raise TypeError(
                        f"Expected type {self._ext['strict_type'][name]} for attribute {name}, but got type {type(value)}")
        super().__setattr__(name, value)

    def __str__(self) -> str:
        def walk(v):
            if v is None:
                return None
            elif isinstance(v, dict):
                return {k: walk(v) for k, v in v.items()}
            elif isinstance(v, (float, int)):
                return v
            else:
                return str(v)
        return json.dumps(walk(self.toJson()), indent=2)

    def __repr__(self) -> str:
        return super().__repr__()



from PySide6.QtWidgets import QApplication
from PySide6.QtGui import QFontDatabase
from .main_window import MainWindow
from .graph_editor import GraphEditor
from ..ir import Model
from typing import Union


def entry(irm: Union[Model, None] = None, path: Union[str, None] = None):
    app = QApplication([])
    font = QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
    app.setFont(font)
    mw = MainWindow(irm, path)
    mw.show()
    app.exec()



from typing import TYPE_CHECKING, List, Dict, Union, Set
from collections import deque
from .obj import IRObj

if TYPE_CHECKING:
    from .node import Node
    from .variable import Variable


class Graph(IRObj):
    def __init__(self) -> None:
        super().__init__()

        self._variables: Dict['Variable'] = {}

        self.name: str = ''
        self.doc_string: str = ''
        self._nodes: List['Node'] = []
        self._input: List[str] = []
        self._output: List[str] = []

        self.displayAttr('name')
        self.displayAttr('doc_string')
        self.displayAttr('_input', name='input')
        self.displayAttr('_output', name='output')
        self.displayAttr('nodes')
        self.displayAttr('variables')

        self.var_add_callback = None
        self.node_add_callback = None
        self.node_del_callback = None
        self.var_mark_input_callback = None
        self.var_mark_output_callback = None
        self.var_unmark_input_callback = None
        self.var_unmark_output_callback = None

    def autoMarkInput(self):
        self._input.extend(
            [k for k, v in self._variables.items() if v.canBeInput])

    def markInput(self, v: Union[str, 'Variable'], run_callback=True):
        if not isinstance(v, str):
            v = v.name
        self.getVariable(v)
        assert v not in self._input
        self._input.append(v)
        if run_callback and self.var_mark_input_callback is not None:
            self.var_mark_input_callback(self.getVariable(v))

    def unMarkInput(self, v: Union[str, 'Variable'], run_callback=True):
        if not isinstance(v, str):
            v = v.name
        assert v in self._input, v
        self._input.remove(v)
        if run_callback and self.var_unmark_input_callback is not None:
            self.var_unmark_input_callback(self.getVariable(v))

    def markOutput(self, v: Union[str, 'Variable'], run_callback=True):
        if not isinstance(v, str):
            v = v.name
        self.getVariable(v)
        assert v not in self._output
        self._output.append(v)
        if run_callback and self.var_mark_output_callback is not None:
            self.var_mark_output_callback(self.getVariable(v))

    def unMarkOutput(self, v: Union[str, 'Variable'], run_callback=True):
        if not isinstance(v, str):
            v = v.name
        assert v in self._output, v
        self._output.remove(v)
        if run_callback and self.var_unmark_output_callback is not None:
            self.var_unmark_output_callback(self.getVariable(v))

    @property
    def nodes(self):
        return [n for n in self._nodes]

    @property
    def variables(self):
        return tuple(self._variables.values())

    @property
    def input(self):
        return [v for v in self._variables.values() if v.name in self._input]

    @property
    def output(self):
        return [v for v in self._variables.values() if v.name in self._output]

    def addNode(self, node: str, op_type: str) -> 'Node':
        from .node import Node
        if isinstance(node, str):
            node = Node(self, node, op_type)
            self._nodes.append(node)
        else:
            assert isinstance(node, Node)
            node.graph = self
            self._nodes.append(node)
        if self.node_add_callback is not None:
            self.node_add_callback(node)
        return node

    def delNode(self, node: 'Node'):
        if node in self._nodes:
            node.removeFromGraph()
            if self.node_del_callback is not None:
                self.node_del_callback(node)

    def addVariable(self, v: Union[str, 'Variable']) -> 'Variable':
        from .variable import Variable
        if isinstance(v, str):
            v = Variable(self, v)
        assert isinstance(v, Variable)
        assert v.name not in self._variables
        self._variables[v.name] = v
        if self.var_add_callback is not None:
            self.var_add_callback(v)
        return v

    def getVariable(self, name: str, auto_create=True) -> 'Variable':
        assert isinstance(name, str)
        if auto_create and name not in self._variables:
            return self.addVariable(name)
        else:
            assert name in self._variables, name
            return self._variables[name]

    def hasVariable(self, v: 'Variable') -> bool:
        if v.name not in self._variables:
            return False
        return self._variables[v.name] == v

    def delVariable(self, v: 'Variable'):
        if isinstance(v, str):
            name = v
        else:
            name = v.name
        if name in self._variables and v is self._variables.get(name):
            v = self.getVariable(v)
            v.removeFromGraph()

    def topoSort(self):
        visited = {}
        nbinputs = {}
        wklist = deque()
        for n in self.getField('nodes').val:
            visited[n] = False
            nbinputs[n] = len(n.prevNodes)
            if nbinputs[n] == 0:
                wklist.append(n)

        sorted = []
        for _ in range(len(visited)):
            if len(wklist) == 0:
                break
            n = wklist.popleft()
            for next_n in n.nextNode:
                nbinputs[next_n] -= 1
                if nbinputs[next_n] == 0:
                    wklist.append(next_n)
            visited[n] = True
            sorted.append(n)
        if all(visited.values()):
            self.getField('nodes').val = sorted
        else:
            raise RuntimeError(
                f'TopoSort error: {[n for n in visited if visited[n] == False]}')

    def _emit_node_input_change(self, node: 'Node', old_ins: Set[str], new_ins: Set[str]):
        for i in old_ins:
            v = self.getVariable(i, False)
            v._dst_nodes.remove(node)
        for i in new_ins:
            v = self.getVariable(i)
            v._dst_nodes.add(node)

    def _emit_node_output_change(self, node: 'Node', old_outs: Set[str], new_outs: Set[str]):
        for i in old_outs:
            v = self.getVariable(i, False)
            v._src_nodes.remove(node)
        for i in new_outs:
            v = self.getVariable(i)
            v._src_nodes.add(node)



from onnxeditor.ir.port import OnnxImport, OnnxExport
from onnxeditor.ir.opt import pass_const_to_var
import onnx.checker
import onnx
import sys

assert len(sys.argv) == 3

imp = OnnxImport(pass_const_to_var)
exp = OnnxExport()

m = onnx.load(sys.argv[1])

onnx.checker.check_model(m)

m = imp(m)

m = exp(m)

onnx.save(m, sys.argv[2])

onnx.checker.check_model(m)



from typing import TYPE_CHECKING, Dict, Any, Set, List, Union, Tuple
from types import FunctionType
from enum import Enum, auto
import numpy as np
import json
from .obj import IRObj
import abc
import copy

if TYPE_CHECKING:
    from .node import Node
    from .graph import Graph


class TensorType(Enum):
    kNone = 'None'
    kBOOL = 'Bool'
    kFP16 = 'Float16'
    kFP32 = 'Float32'
    kFP64 = 'Float64'
    kINT8 = 'Int8'
    kINT32 = 'Int32'
    kINT64 = 'Int64'

    @staticmethod
    def fromNumpy(np_dt):
        np2dt = {
            np.float32: TensorType.kFP32,
            np.float64: TensorType.kFP64,
            np.int32: TensorType.kINT32,
            np.int64: TensorType.kINT64,
        }
        if isinstance(np_dt, np.dtype):
            np_dt = np_dt.type
        assert np_dt in np2dt, (np_dt,)
        return np2dt[np_dt]

    def toNumpy(self):
        dt2np = {
            TensorType.kFP32: np.float32,
            TensorType.kFP64: np.float64,
            TensorType.kINT32: np.int32,
            TensorType.kINT64: np.int64,
        }
        assert self in dt2np, f'not handled {self} yet'
        return dt2np[self]


class DataBase(abc.ABC):
    @property
    @abc.abstractmethod
    def type(self) -> np.dtype: ...

    @property
    @abc.abstractmethod
    def type(self) -> Tuple: ...

    @property
    def location(self):
        return None

    @abc.abstractmethod
    def getNp(self) -> np.ndarray: ...


class NativeData(DataBase):
    def __init__(self, data) -> None:
        self._data = data
        super().__init__()

    @property
    def type(self):
        return self._data.dtype

    @property
    def shape(self):
        return self._data.shape

    def getNp(self) -> np.ndarray:
        return self._data


class Variable(IRObj):
    def __init__(self, graph: Union['Graph', None], name: str, shape: list = list(), type: Union[TensorType, np.dtype] = TensorType.kNone, data: Union[None, np.ndarray, DataBase] = None) -> None:
        super().__init__()

        assert isinstance(name, str)
        self._name: str = name
        self._shape: list = shape
        self._type: TensorType = None
        self._data: DataBase = None
        self.doc_string = ''

        self.type = type
        self.data = data

        self._src_nodes: Set['Node'] = set()
        self._dst_nodes: Set['Node'] = set()

        self._graph: Union['Graph', None] = graph

        self._marked_variable = False

        self.displayAttr('name')
        self.displayAttr('shape')
        self.displayAttr('type')
        self.displayAttr('_data', name='data')
        self.displayAttr('doc_string')

        self._ext['bind_gedge'] = None
        self._ext['bind_gnode_src'] = None
        self._ext['bind_gnode_dst'] = None
        self._ext['bind_gnode_last'] = None

    @property
    def graph(self):
        return self._graph

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, v):
        assert self._graph is None
        self._name = v

    @property
    def data(self):
        return self._data

    @property
    def location(self):
        return self._data.location

    @data.setter
    def data(self, d):
        if isinstance(d, np.ndarray):
            d = NativeData(d)
        if d is None:
            self._data = None
        else:
            assert isinstance(d, DataBase)
            self._data = d

    @property
    def type(self):
        if self._data is None:
            return self._type
        else:
            return TensorType.fromNumpy(self._data.type)

    @type.setter
    def type(self, t):
        assert self._data is None, f'{self.name} already is a constant, can not set type explicitly'
        if t is None or isinstance(t, TensorType):
            self._type = t
        else:
            self._type = TensorType.fromNumpy(t)

    @property
    def shape(self):
        if self._data is None:
            return self._shape
        else:
            return self._data.shape

    @shape.setter
    def shape(self, s):
        assert self._data is None
        self._shape = tuple(s)

    @property
    def src(self):
        return self._src_nodes.copy()

    @property
    def dst(self):
        return self._dst_nodes.copy()

    @property
    def used(self):
        return (len(self._src_nodes) + len(self._dst_nodes)) > 0 or self._graph is None

    @property
    def canBeInput(self):
        return not self.isConstant and len(self._src_nodes) == 0 and self.used and self._graph is not None

    @property
    def maybeOutput(self):
        return len(self._dst_nodes) == 0 and self.used and self._graph is not None

    @property
    def isInput(self):
        if self._graph is None:
            return False
        else:
            return self in self._graph.input

    @property
    def isOutput(self):
        if self._graph is None:
            return False
        else:
            return self in self._graph.output

    @property
    def isConstant(self):
        return self.data is not None and not self._marked_variable

    def markVariable(self):
        self._marked_variable = True

    def removeFromGraph(self):
        assert not self.used
        del self._graph._variables[self.name]
        self._graph = None

    def copyOut(self):
        new = copy.copy(self)
        new._graph = None
        new._src_nodes = set()
        new._dst_nodes = set()
        return copy.deepcopy(new)

    def markInput(self, *args, **kwargs):
        assert self._graph is not None
        self._graph.markInput(self, *args, **kwargs)

    def unMarkInput(self, *args, **kwargs):
        assert self._graph is not None
        self._graph.unMarkInput(self.name, *args, **kwargs)

    def markOutput(self, *args, **kwargs):
        assert self._graph is not None
        self._graph.markOutput(self, *args, **kwargs)

    def unMarkOutput(self, *args, **kwargs):
        assert self._graph is not None
        self._graph.unMarkOutput(self.name, *args, **kwargs)



from typing import Optional, Union
import PySide6.QtCore
from PySide6.QtWidgets import QDialog, QWidget, QDialogButtonBox, QFileDialog, QMessageBox
from PySide6.QtCore import Qt, Slot, Signal
from ....ir import Variable, TensorType
from .ui_datainspector import Ui_DataInspector
import re
import os
import numpy as np


class DataInspector(QDialog):
    def __init__(self, vir: Union[Variable, None] = None, parent: Union[QWidget, None] = None, display_name: bool = True) -> None:
        super().__init__(parent)
        self._ui = Ui_DataInspector()
        self._ui.setupUi(self)
        
        if not display_name:
            self._ui.name_label.hide()
            self._ui.var_name.hide()

        self.handle_map = {
            'npy': (' single tensor', self.handle_npy),
        }

        self._ui.file_btn.clicked.connect(self.file_brow)
        self._ui.load_btn.clicked.connect(self.load_file)
        self._ui.dump_to_btn.clicked.connect(self.dump_file)

        self._data: Union[np.ndarray, None] = None
        
        if vir is not None:
            self._ui.var_name.setText(vir.name)
            self._data = vir.data.getNp()
        
        self.flush_data_info()

    def flush_data_info(self):
        b = self._ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok)
        if self._data is None:
            s = f'No Data Loaded'
            self._ui.dump_to_btn.setDisabled(True)
            b.setDisabled(True)
        else:
            s = f'Type: {self._data.dtype}\nShape: {list(self._data.shape)}'
            self._ui.dump_to_btn.setDisabled(False)
            b.setDisabled(False)
        self._ui.data_info.setText(s)
        
    def file_brow(self):
        path = QFileDialog.getOpenFileName(self, "open data file", "/", ' '.join([f'*.{k}' for k in self.handle_map]))
        if path is None or len(path[0]) == 0:
            return
        else:
            self._ui.path_edit.setText(path)
    
    @Slot()
    def dump_file(self):
        path = QFileDialog.getSaveFileName(self, "save data file", self._path, '*.npy')
        if path is None or len(path[0]) == 0:
            return
        else:
            try:
                np.save(path, self._data)
            except Exception as e:
                QMessageBox.warning(self, "dump error", str(e))
    
    @Slot()
    def load_file(self):
        path = self._ui.path_edit.text()
        suffix = path.split('.')[-1]
        print(suffix)
        if suffix not in self.handle_map:
            t = ',\n'.join([f'*.{k}' for k in self.handle_map])
            QMessageBox.warning(self, "suffix not support", f'we not support this suffix, we only support:\n[\n{t}\n]')
            self._data = None
        else:
            try:
                self._data = self.handle_map[suffix][1](path)
            except Exception as e:
                QMessageBox.warning(self, "data file load error", str(e))
                self._data = None
        self.flush_data_info()

    def handle_npy(self, path):
        self._data = np.load(path)
        
    def getRet(self):
        return self._data, self._ui.var_name.text()



from ...ir import Variable
from typing import List, TYPE_CHECKING, Any
from PySide6.QtCore import Signal, Qt, QRectF, QPointF, Slot
from PySide6.QtGui import QPainter, QColor, QPen, QFont, QFontMetrics, QPainterPath, QPainterPathStroker
from PySide6.QtWidgets import QGraphicsItem, QGraphicsObject, QGraphicsSceneHoverEvent, QGraphicsSceneMouseEvent, QStyleOptionGraphicsItem, QWidget, QGraphicsPathItem
from typing import Union

if TYPE_CHECKING:
    from .node import GraphNode


class GraphEdge(QGraphicsObject):
    def __init__(self, ir: Variable, parent=None):
        super().__init__()
        self._ir: Variable = ir

        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, True)
        self.setAcceptHoverEvents(True)
        self.setZValue(0)

        self._hovered = False

        self._src_pts: List[QPointF] = []
        self._dst_pts: List[QPointF] = []
        self._path: QPainterPath = QPainterPath()

    @property
    def ir(self):
        return self._ir

    @Slot(list)
    def needUpdate(self, nodes: List['GraphNode']):
        assert len(nodes) == 1
        node = nodes[0]
        src_obj = [n.read_ext('bind_gnode') for n in self._ir.src if n.read_ext(
            'bind_gnode') is not None]
        dst_obj = [n.read_ext('bind_gnode') for n in self._ir.dst if n.read_ext(
            'bind_gnode') is not None]
        if self._ir.isInput:
            gn = self._ir.read_ext('bind_gnode_src')
            assert gn is not None
            src_obj += [gn]
        if self._ir.isOutput:
            gn = self._ir.read_ext('bind_gnode_dst')
            assert gn is not None
            dst_obj += [gn]
        assert all([v is not None for v in src_obj])
        assert all([v is not None for v in dst_obj])
        self.drawEdge(src_obj, dst_obj)
        if node not in src_obj + dst_obj:
            node.pos_move.disconnect(self.needUpdate)
            node.io_change.disconnect(self.needUpdate)

    def drawEdge(self, src_obj: List[QGraphicsItem], dst_obj: List[QGraphicsItem]):
        self._src_pts.clear()
        for o in src_obj:
            pos = o.pos()
            rect = o.boundingRect()
            self._src_pts.append(
                QPointF(rect.left() + rect.width() / 2, rect.bottom()) + pos)
            # print('src', o._ir.name, pos, rect, self._src_pts[-1])
        self._dst_pts.clear()
        for o in dst_obj:
            pos = o.pos()
            rect = o.boundingRect()
            self._dst_pts.append(
                QPointF(rect.left() + rect.width() / 2, rect.top()) + pos)
            # print('dst', o._ir.name, pos, rect, self._src_pts[-1])
        self._path.clear()
        for s in self._src_pts:
            for d in self._dst_pts:
                self._path.moveTo(s)
                c1 = QPointF(s.x(), (s.y() + d.y()) / 2)
                c2 = QPointF(d.x(), (s.y() + d.y()) / 2)
                self._path.cubicTo(c1, c2, d)
        self.prepareGeometryChange()
        self.update()

    def shape(self) -> QPainterPath:
        ps = QPainterPathStroker()
        pen = QPen()
        if self._hovered or self.isSelected():
            pen.setWidthF(2 * 3.0)
        else:
            pen.setWidthF(3.0)
        ps.setCapStyle(pen.capStyle())
        ps.setWidth(pen.widthF() + 10)
        ps.setJoinStyle(pen.joinStyle())
        ps.setMiterLimit(pen.miterLimit())
        p = ps.createStroke(self._path)
        for s in self._src_pts:
            p.addEllipse(s, 3.0, 3.0)
        for d in self._dst_pts:
            p.addEllipse(d, 5.0, 5.0)
        return p

    def boundingRect(self) -> QRectF:
        return self.shape().boundingRect()

    def paint(self, painter: QPainter, option: QStyleOptionGraphicsItem, widget: Union[QWidget, None] = ...) -> None:
        painter.save()
        # line hovered
        if self._hovered or self.isSelected():
            p = QPen()
            p.setWidthF(2 * 3.0)
            p.setColor(QColor(255, 165, 0) if self.isSelected()
                       else QColor(224, 255, 255))
            painter.setPen(p)
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.drawPath(self._path)
        # line body
        p = QPen()
        p.setWidthF(3.0)
        if len(self._src_pts) > 1:
            p.setColor(QColor(248, 4, 2))
        else:
            p.setColor(QColor(100, 100, 100) if self.isSelected()
                       else QColor(0, 139, 139))
        painter.setPen(p)
        painter.setBrush(Qt.BrushStyle.NoBrush)
        painter.drawPath(self._path)
        # ellipse hovered
        color = QColor(255, 165, 0) if self.isSelected(
        ) else QColor(224, 255, 255)
        for s in self._src_pts:
            painter.setPen(color)
            painter.setBrush(color)
            painter.drawEllipse(s, 4.0, 4.0)
        for d in self._dst_pts:
            painter.setPen(color)
            painter.setBrush(color)
            painter.drawEllipse(d, 6.0, 6.0)
        # ellipse body
        for s in self._src_pts:
            painter.setPen(QColor(128, 128, 128))
            painter.setBrush(QColor(128, 128, 128))
            painter.drawEllipse(s, 3.0, 3.0)
        for d in self._dst_pts:
            painter.setPen(QColor(128, 128, 128))
            painter.setBrush(QColor(128, 128, 128))
            painter.drawEllipse(d, 5.0, 5.0)
        painter.restore()

    def hoverEnterEvent(self, event: QGraphicsSceneHoverEvent) -> None:
        # print('hoverEnterEvent')
        self._hovered = True
        self.update()
        return super().hoverEnterEvent(event)

    def hoverLeaveEvent(self, event: QGraphicsSceneHoverEvent) -> None:
        # print('hoverLeaveEvent')
        self._hovered = False
        self.update()
        return super().hoverLeaveEvent(event)

    def itemChange(self, change: QGraphicsItem.GraphicsItemChange, value: Any) -> Any:
        if change == QGraphicsItem.GraphicsItemChange.ItemSelectedHasChanged:
            assert isinstance(value, int)
            if value == 1:
                self.setZValue(3)
            else:
                self.setZValue(0)
            self.update()
        return super().itemChange(change, value)



from onnxeditor.ir.base import Model
import numpy as np

m = Model()

m.producer_name = 123

'''
in0 -> |n0| --> |
                |->|n1| -> out0
     |-> |n2| ->|
in1 -|          |
     |-> |n3| ->|
         |    \-> out1
const0 ->|
'''

const0 = m.graph.addVariable('const0')
const0.data = np.array([1, 2, 3])

n0 = m.graph.addNode('n0', 'n0')
n1 = m.graph.addNode('n1', 'n1')
n2 = m.graph.addNode('n2', 'n2')
n3 = m.graph.addNode('n3', 'n3')

n0.input = ['in0']
n0.output = ['n0_out']
n1.input = [
    'n0_out',
    'n2_out',
    'out1',
]
n1.output = ['out0']
n2.input = ['in1']
n2.output = ['n2_out']
n3.input = ['in1', 'const0']
n3.output = ['out1']

m.graph.markOutput('out0')
m.graph.markOutput('out1')

m.graph.autoMarkInput()

print(m)



from ..base import Model
from ..port import OnnxImport

from typing import Tuple


def pass_const_to_var(m: Model) -> Tuple[bool, str]:
    assert isinstance(m, Model)
    g = m.graph
    for n in g.nodes:
        if n.op_type == 'Constant':
            assert len(n.attrs) == 1
            k, v = list(n.attrs.items())[0]
            assert len(n.output) == 1
            out = n.output[0]
            if k == 'value':
                out.data = v.data
            else:
                raise NotImplementedError(
                    f'we not impl the elements type for constant node, the attr is {k} : {v}')
            g.delNode(n)
    return True, 'succ'



from ..base import Model, Graph, Variable, DataBase, Node, TensorType
from typing import TYPE_CHECKING, Union, List
import onnx
import onnx.numpy_helper
import numpy as np

# copy from https://github.com/NVIDIA/TensorRT/blob/release/8.6/tools/onnx-graphsurgeon/onnx_graphsurgeon/importers/onnx_importer.py

# Maps values from the AttributeType enum to their string representations, e.g., {1: "FLOAT"}
ATTR_TYPE_MAPPING = dict(zip(onnx.AttributeProto.AttributeType.values(
), onnx.AttributeProto.AttributeType.keys()))

# Maps an ONNX attribute to the corresponding Python property
ONNX_PYTHON_ATTR_MAPPING = {
    "FLOAT": "f",
    "INT": "i",
    "STRING": "s",
    "TENSOR": "t",
    "GRAPH": "g",
    "FLOATS": "floats",
    "INTS": "ints",
    "STRINGS": "strings",
}


def get_onnx_tensor_shape(onnx_tensor: Union[onnx.ValueInfoProto, onnx.TensorProto]) -> List[int]:
    shape = []
    if isinstance(onnx_tensor, onnx.TensorProto):
        shape = onnx_tensor.dims
    else:
        if onnx_tensor.type.tensor_type.HasField("shape"):
            shape = []
            for dim in onnx_tensor.type.tensor_type.shape.dim:
                if dim.HasField("dim_param"):
                    shape.append(dim.dim_param)
                elif dim.HasField("dim_value"):
                    shape.append(dim.dim_value)
                else:
                    shape.append(None)
    return shape


def get_onnx_tensor_dtype(onnx_tensor: Union[onnx.ValueInfoProto, onnx.TensorProto]) -> np.dtype:
    if isinstance(onnx_tensor, onnx.TensorProto):
        onnx_type = onnx_tensor.data_type
    else:
        onnx_type = onnx_tensor.type.tensor_type.elem_type
    if onnx_type in onnx.mapping.TENSOR_TYPE_TO_NP_TYPE:
        return onnx.mapping.TENSOR_TYPE_TO_NP_TYPE[onnx_type]
    else:
        raise NotImplementedError(f'{onnx_type} not handled')


class LazyData(DataBase):
    def __init__(self, tensor, location) -> None:
        self._tensor = tensor
        self._data = tensor
        self._location = location
        self._type = get_onnx_tensor_dtype(tensor)
        self._shape = tuple(get_onnx_tensor_shape(tensor))
        super().__init__()

    @property
    def type(self):
        return self._type

    @property
    def shape(self):
        return self._shape

    @property
    def location(self):
        return self._location

    @property
    def tensor(self):
        return self._tensor

    def getNp(self) -> np.ndarray:
        if not isinstance(self._data, np.ndarray):
            self._data = np.array(onnx.numpy_helper.to_array(self._data))
            assert self._type == self._data.dtype
            assert self._shape == self._data.shape
        return self._data


class OnnxImport:
    def __init__(self, pass_fns: list = list()) -> None:
        if not isinstance(pass_fns, (list, tuple)):
            pass_fns = [pass_fns]
        self._pass_fns = pass_fns

    def __call__(self, m) -> Model:
        irm = Model()

        if isinstance(m, str):
            m = onnx.load(m)

        assert isinstance(m, onnx.ModelProto), (type(m), m)

        self.parse_model(m, irm)

        for fn in self._pass_fns:
            ret = fn(irm)
            assert ret[0], ret[1]

        return irm

    def parse_model(self, src: onnx.ModelProto, dst: Model):
        dst.opset_import = {}
        for v in src.opset_import:
            assert v.domain not in dst.opset_import
            dst.opset_import[v.domain] = v.version
        for field in ['ir_version', 'producer_name', 'producer_version', 'domain', 'model_version', 'doc_string']:
            setattr(dst, field, getattr(src, field) if src.HasField(
                field) else None)
        self.parse_graph(src.graph, dst.graph)

    def parse_graph(self, src: onnx.GraphProto, dst: Graph):
        dst.name = src.name
        dst.doc_string = src.doc_string if src.HasField("doc_string") else None
        for v in src.output:
            irv = self.parse_value_info(v, dst)
            dst.markOutput(irv)
        const_var_name = []
        for v in src.initializer:
            irv = self.parse_tensor(v, dst)
            const_var_name.append(irv.name)
        for v in src.value_info:
            self.parse_value_info(v, dst)
        for n in src.node:
            self.parse_node(n, dst)
        for v in src.input:
            irv = self.parse_value_info(v, dst)
            if irv.name not in const_var_name:
                dst.markInput(irv)
        return dst

    def parse_value_info(self, src: onnx.ValueInfoProto, dst: Graph):
        v = dst.getVariable(src.name)
        if not v.isConstant:
            try:
                v.type = get_onnx_tensor_dtype(src)
            except NotImplementedError as e:
                print(f'get err when handle dtype, skiped: {e}')
                v.type = TensorType.kNone
            v.shape = get_onnx_tensor_shape(src)
        return v

    def parse_tensor(self, src: onnx.TensorProto, dst: Union[Graph, None]):
        if dst is None:
            v = Variable(None, src.name)
        else:
            v = dst.getVariable(src.name)
        data_location = int(src.data_location) if src.HasField(
            "data_location") else None
        v.data = LazyData(src, data_location)
        return v

    def parse_node(self, src: onnx.NodeProto, dst: Graph):
        n = dst.addNode(src.name, src.op_type)
        n.input = [str(v) for v in src.input]
        n.output = [str(v) for v in src.output]
        n.domain = src.domain if src.HasField("domain") else None
        n.doc_string = src.doc_string if src.HasField("doc_string") else None
        n.clearAttr()
        for attr in src.attribute:
            if attr.type in ATTR_TYPE_MAPPING:
                attr_str = ATTR_TYPE_MAPPING[attr.type]
                if attr_str in ONNX_PYTHON_ATTR_MAPPING:
                    assert attr.name not in n.attrs
                    processed = getattr(
                        attr, ONNX_PYTHON_ATTR_MAPPING[attr_str])
                    if attr_str == "STRING":
                        processed = processed.decode()
                    elif attr_str == "TENSOR":
                        processed = self.parse_tensor(processed, None)
                    elif attr_str == "GRAPH":
                        g = Graph()
                        self.parse_graph(processed, g)
                        processed = g
                    elif attr_str == "FLOATS" or attr_str == "INTS":
                        processed = list(processed)
                    elif attr_str == "STRINGS":
                        processed = [p.decode() for p in processed]
                    n.attrs[attr.name] = processed
                elif attr_str == 'UNDEFINED':
                    print(
                        f'warning: UNDEFINED attr got in node: {n.name}, op_type: {n.op_type}')
                    pass
                else:
                    raise KeyError(
                        f'attr not handled, maybe new ir: {attr_str}, node: {n.name}, op_type: {n.op_type}')
            else:
                raise KeyError(f'attr not handled, maybe new ir: {attr.type}')
        return n



from onnxeditor.ir.port import OnnxImport, OnnxExport
import onnx.checker
import onnx

import sys

assert len(sys.argv) == 3

imp = OnnxImport()
exp = OnnxExport()

m = onnx.load(sys.argv[1])

# onnx.checker.check_model(m)

m = imp(m)

m = exp(m)

onnx.save(m, sys.argv[2])

# onnx.checker.check_model(m)



from typing import Dict
from .obj import IRObj
from .graph import Graph


class Model(IRObj):
    def __init__(self) -> None:
        super().__init__()

        self.ir_version: int = 0
        self.opset_import: Dict[str, int] = {}
        self.producer_name: str = ''
        self.producer_version: str = ''
        self.domain: str = ''
        self.model_version: int = 0
        self.doc_string: str = ''
        self.graph: Graph = Graph()

        self.displayAttr('ir_version')
        self.displayAttr('opset_import')
        self.displayAttr('producer_name')
        self.displayAttr('producer_version')
        self.displayAttr('domain')
        self.displayAttr('model_version')
        self.displayAttr('doc_string')
        self.displayAttr('graph')



import os
# import sys

try:
    from .ui_iosummary import Ui_IOSummary
except ImportError:
    dir = os.path.dirname(__file__)
    # sys.path.append(dir)
    target = os.path.join(dir, 'ui_iosummary.py')
    if not os.path.exists(target):
        os.system(
            f'cd {dir} && pyside6-uic iosummary.ui -o ui_iosummary.py')

from .ui_iosummary import Ui_IOSummary
from .iosummary import IOSummary



from typing import Optional, Any, Union
import PySide6.QtCore
from PySide6.QtWidgets import QDialog, QWidget, QListWidget, QListWidgetItem, QAbstractItemView, QHeaderView, QComboBox, QTableWidget, QTableWidgetItem, QPushButton, QLineEdit, QStackedLayout, QCompleter
from PySide6.QtCore import Qt, Slot, Signal
from ....ir import Node, Variable, NativeData, Graph
from .ui_nodesummary import Ui_NodeSummary
from ..datainspector import DataInspector
import re
import numpy as np


class AttrValue(QWidget):
    def __init__(self, v, parent: Union[QWidget, None] = None) -> None:
        super().__init__(parent)
        self._edit = QLineEdit()
        self._edit.setPlaceholderText("-")
        self._btn = QPushButton('Edit Tensor')

        self._data = None
        if isinstance(v, Variable):
            self._data = v.data
        if isinstance(v, (list, tuple)):
            self._edit.setText(','.join([str(vv) for vv in v]))
        else:
            self._edit.setText(str(v))

        self._layout = QStackedLayout()
        self._layout.addWidget(self._edit)
        self._layout.addWidget(self._btn)

        self.setLayout(self._layout)

        self._btn.clicked.connect(self.set_data)

    @Slot(str)
    def needSwitch(self, v):
        if v == 'Tensor':
            self.showBtn()
        else:
            self.showEdit()

    def showEdit(self):
        self._layout.setCurrentWidget(self._edit)

    def showBtn(self):
        self._layout.setCurrentWidget(self._btn)

    def parseElem(self, txt):
        if re.fullmatch(f'^-?\d+$', txt) is not None:
            return int(txt)
        elif re.fullmatch(f'^-?\d+\.\d*$', txt) is not None:
            return float(txt)
        else:
            return str(txt)

    def getVal(self):
        w = self._layout.currentWidget()
        if w is self._edit:
            txt = self._edit.text()
            if len(txt) == 0:
                return ''
            if re.fullmatch(f'^-?\d+$', txt) is not None:
                return int(txt)
            elif re.fullmatch(f'^-?\d+\.\d*$', txt) is not None:
                return float(txt)
            elif ',' in txt and re.fullmatch(f'^((-?[0-9a-zA-Z]+),?)*$', txt) is not None:
                ret = re.findall(r'(-?[0-9a-zA-Z]+)', txt)
                return [self.parseElem(r) for r in ret]
            else:
                return txt
        else:
            if self._data is None:
                return np.array([])
            return self._data

    @Slot()
    def set_data(self):
        dialog = DataInspector(display_name=False)
        dialog.setWindowTitle('Set Data Attribute')
        ret = dialog.exec()
        if ret == QDialog.DialogCode.Accepted:
            ret = dialog.getRet()[0]
            self._data = NativeData(ret)


class NodeSummary(QDialog):
    def __init__(self, nir: Union[Node, None] = None, g: Union[Graph, None] = None, parent: Union[QWidget, None] = None) -> None:
        super().__init__(parent)
        self._ui = Ui_NodeSummary()
        self._ui.setupUi(self)

        if g is None:
            if nir is not None:
                if nir.graph is not None:
                    g = nir.graph

        if g is not None:
            self._cmp = QCompleter([v.name for v in g.variables], self)
        else:
            self._cmp = None

        header = ['name', 'type', 'value']
        self._ui.inputs_list_widget.setAlternatingRowColors(True)
        self._ui.attr_tabel_widget.setColumnCount(len(header))
        self._ui.attr_tabel_widget.verticalHeader().hide()
        self._ui.attr_tabel_widget.verticalHeader().setDefaultSectionSize(15)
        self._ui.attr_tabel_widget.setHorizontalHeaderLabels(header)
        self._ui.attr_tabel_widget.setSelectionBehavior(
            QAbstractItemView.SelectionBehavior.SelectRows)
        self._ui.attr_tabel_widget.setSelectionMode(
            QAbstractItemView.SelectionMode.SingleSelection)
        self._ui.attr_tabel_widget.setAlternatingRowColors(True)
        self._ui.attr_tabel_widget.horizontalHeader().setSectionResizeMode(0,
                                                                           QHeaderView.ResizeMode.Stretch)
        self._ui.attr_tabel_widget.horizontalHeader().setSectionResizeMode(1,
                                                                           QHeaderView.ResizeMode.Stretch)
        self._ui.attr_tabel_widget.horizontalHeader().setSectionResizeMode(2,
                                                                           QHeaderView.ResizeMode.Stretch)

        self._ui.inputs_add.clicked.connect(
            lambda _: self.addItem(self._ui.inputs_list_widget, ""))
        self._ui.inputs_del.clicked.connect(
            lambda _: self.delItem(self._ui.inputs_list_widget))
        self._ui.outputs_add.clicked.connect(
            lambda _: self.addItem(self._ui.outputs_list_widget, ""))
        self._ui.outputs_del.clicked.connect(
            lambda _: self.delItem(self._ui.outputs_list_widget))
        self._ui.attr_add.clicked.connect(
            lambda _: self.addRow("", '', False))
        self._ui.attr_del.clicked.connect(lambda _: self.delRow())

        if nir is not None:
            self._ui.name_edit.setText(nir.name)
            self._ui.op_type_edit.setText(nir.op_type)
            for i in nir.input:
                self.addItem(self._ui.inputs_list_widget, i.name)
            for o in nir.output:
                self.addItem(self._ui.outputs_list_widget, o.name)
            for k, v in nir.attrs.items():
                self.addRow(k, v, isinstance(v, Variable))

    def addItem(self, lw: QListWidget, name: str):
        item = QListWidgetItem()
        edit = QLineEdit(name)
        edit.setPlaceholderText('-')
        if self._cmp is not None:
            edit.setCompleter(self._cmp)
        lw.addItem(item)
        lw.setItemWidget(item, edit)
        lw.setCurrentItem(item)

    def delItem(self, lw: QListWidget):
        for item in lw.selectedItems():
            lw.removeItemWidget(item)

    def addRow(self, key: str, value: Any, istensor: bool = False):
        row = self._ui.attr_tabel_widget.rowCount()
        self._ui.attr_tabel_widget.insertRow(row)
        # type
        box = QComboBox()
        box.addItems(['Scalar(s)', 'Tensor'])
        box.setCurrentIndex(1 if istensor else 0)
        self._ui.attr_tabel_widget.setCellWidget(row, 1, box)
        # name
        # item = QTableWidgetItem(key)
        # item.setFlags(item.flags() | Qt.ItemFlag.ItemIsEditable)
        # self._ui.attr_tabel_widget.setItem(row, 0, item)
        item = QLineEdit(key)
        item.setPlaceholderText("-")
        self._ui.attr_tabel_widget.setCellWidget(row, 0, item)
        # value
        item = AttrValue(value)
        if istensor:
            item.showBtn()
        else:
            item.showEdit()
        self._ui.attr_tabel_widget.setCellWidget(row, 2, item)
        box.currentTextChanged.connect(item.needSwitch)

    def delRow(self):
        row_id = [item.row()
                  for item in self._ui.attr_tabel_widget.selectedItems()]
        row_id = sorted(row_id, reverse=True)
        for id in row_id:
            self._ui.attr_tabel_widget.removeRow(id)

    def getRet(self):
        ins_w = self._ui.inputs_list_widget
        outs_w = self._ui.outputs_list_widget
        attrs_w = self._ui.attr_tabel_widget
        ret = {
            'name': self._ui.name_edit.text(),
            'op_type': self._ui.op_type_edit.text(),
            'inputs': [ins_w.itemWidget(ins_w.item(i)).text() for i in range(ins_w.count())],
            'output': [outs_w.itemWidget(outs_w.item(i)).text() for i in range(outs_w.count())],
            'attrs': {}
        }
        for r in range(attrs_w.rowCount()):
            # k = attrs_w.item(r, 0).text()
            k = attrs_w.cellWidget(r, 0).text()
            # t_w = attrs_w.cellWidget(r, 1)
            v_w = attrs_w.cellWidget(r, 2)
            assert k not in ret['attrs']
            ret['attrs'][k] = v_w.getVal()
        return ret



from typing import Optional, Any, Union
import PySide6.QtCore
from PySide6.QtWidgets import QDialog, QWidget, QHBoxLayout, QVBoxLayout, QLabel, QLineEdit, QTextEdit
from PySide6.QtCore import Qt, Slot, Signal
from ....ir import Model
import re
import numpy as np
import json


class ModelEditor(QDialog):
    def __init__(self, irm: Model, parent: Union[QWidget, None] = None) -> None:
        super().__init__(parent)

        vl = QVBoxLayout()

        self._key = {'ir_version': (QLineEdit(), int, 0), 'producer_name': (QLineEdit(), str, ''), 'producer_version': (QLineEdit(), str, ''),
                     'domain': (QLineEdit(), str, ''), 'model_version': (QLineEdit(), int, 0), 'doc_string': (QLineEdit(), str, 0)}
        for k, v in self._key.items():
            hl = QVBoxLayout()
            hl.addWidget(QLabel(f'{k}:'))
            t = getattr(irm, k)
            if not isinstance(t, v[1]):
                t = v[1](v[2])
            v[0].setText(str(t))
            hl.addWidget(v[0])
            vl.addLayout(hl)

        hl = QVBoxLayout()
        hl.addWidget(QLabel('opset_import:'))
        self._opset_import = QTextEdit()
        self._opset_import.setText(json.dumps(irm.opset_import, indent=2))
        hl.addWidget(self._opset_import)
        vl.addLayout(hl)

        self.setLayout(vl)

    def getRet(self):
        ret = {k: v[1](v[0].text()) for k, v in self._key.items()}
        ret['opset_import'] = json.loads(self._opset_import.toPlainText())
        return ret



from typing import Optional, Union
from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QDialog, QWidget, QDialogButtonBox, QGraphicsItem, QListWidgetItem
from PySide6.QtCore import Qt, Slot, Signal, QObject
from ....ir import Graph, Node, Variable
from .ui_findbar import Ui_FindBar
import re


class FindBar(QDialog):
    centerOn = Signal(QGraphicsItem)

    def __init__(self, gir: Graph, parent: Union[QWidget, None] = None) -> None:
        super().__init__(parent)
        self._ui = Ui_FindBar()
        self._ui.setupUi(self)

        self._ir = gir

        self._ui.btn_find.clicked.connect(self.doFind)

        self._ui.filter_node.setChecked(True)
        self._ui.filter_io.setChecked(True)
        self._ui.filter_var.setChecked(True)

        self._ui.ret_list.itemDoubleClicked.connect(self.onItemDoubleClicked)

        self.setWindowTitle('Find')

    def addItem(self, name: str, node: QGraphicsItem):
        item = QListWidgetItem(name)
        assert node is not None
        ir = node.ir
        if isinstance(ir, Node):
            item.setIcon(QIcon(":/img/node.png"))
        elif isinstance(ir, Variable):
            if not ir.used:
                item.setIcon(QIcon(":/img/unused.png"))
            else:
                if ir.isInput:
                    item.setIcon(QIcon(":/img/input.png"))
                elif ir.isOutput:
                    item.setIcon(QIcon(":/img/output.png"))
                elif ir.isConstant:
                    item.setIcon(QIcon(":/img/tensor4.png"))
                else:
                    item.setIcon(QIcon(":/img/var.png"))
        item.setData(Qt.ItemDataRole.UserRole, node)
        self._ui.ret_list.addItem(item)

    def clearItem(self):
        self._ui.ret_list.clear()

    @Slot()
    def doFind(self):
        self.clearItem()

        name = self._ui.le_name.text()
        type = self._ui.find_mod.currentText()

        if type == 'Has':
            def fn(s):
                return name in s
        elif type == 'StartWith':
            def fn(s):
                return s.startswith(name)
        elif type == 'EndsWith':
            def fn(s):
                return s.endswith(name)
        elif type == 'Regex':
            def fn(s):
                return re.fullmatch(name, s) is not None
        else:
            raise RuntimeError(f'Unknown find type: {type}')

        if self._ui.filter_node.isChecked():
            for n in self._ir.nodes:
                if fn(n.name):
                    self.addItem(n.name, n.read_ext('bind_gnode'))
        if self._ui.filter_io.isChecked():
            for v in self._ir.input:
                if fn(v.name):
                    self.addItem(v.name, v.read_ext('bind_gnode_src'))
            for v in self._ir.output:
                if fn(v.name):
                    self.addItem(v.name, v.read_ext('bind_gnode_dst'))
        if self._ui.filter_var.isChecked():
            for v in self._ir.variables:
                if fn(v.name):
                    self.addItem(v.name, v.read_ext('bind_gedge'))

    @Slot(QListWidgetItem)
    def onItemDoubleClicked(self, item: QListWidgetItem):
        it = item.data(Qt.ItemDataRole.UserRole)
        if it is not None:
            self.centerOn.emit(it)



import onnx
import onnx.helper
from ..base import Model, Graph, Variable, Node, TensorType
from .imp import LazyData
import numpy as np


class OnnxExport:
    def __call__(self, ir: Model, path: str = None) -> onnx.ModelProto:
        assert isinstance(ir, Model)
        m = self.parse_model(ir)
        if path is not None:
            onnx.save(m, path)
        return m

    def parse_model(self, ir: Model):
        g = self.parse_graph(ir.graph)
        kwargs = {'opset_imports': []}
        for k, v in ir.opset_import.items():
            kwargs['opset_imports'].append(
                onnx.OperatorSetIdProto(domain=k, version=v))
        for field in ['ir_version', 'producer_name', 'producer_version', 'domain', 'model_version', 'doc_string']:
            v = getattr(ir, field)
            if v is not None:
                kwargs[field] = v
        m = onnx.helper.make_model(g, **kwargs)
        return m

    def parse_graph(self, ir: Graph):
        nodes = [self.parse_node(n) for n in ir.nodes]
        inputs = [self.parse_value_info(v) for v in ir.input]
        outputs = [self.parse_value_info(v) for v in ir.output]
        initializer = [self.parse_tensor(
            v) for v in ir.variables if v.isConstant and v.used]
        value_info = [self.parse_value_info(
            v) for v in ir.variables if not v.isConstant and v.used]
        value_info = [v for v in value_info if v is not None]
        inputs.extend([self.parse_value_info(v)
                      for v in ir.variables if v.isConstant and v.used])
        g = onnx.helper.make_graph(
            nodes=nodes,
            name=ir.name,
            inputs=inputs,
            outputs=outputs,
            initializer=initializer,
            doc_string=ir.doc_string,
            value_info=value_info,
        )
        return g

    def parse_value_info(self, ir: Variable):
        if ir.type is None or ir.type is TensorType.kNone:
            t = 0
        else:
            t = onnx.helper.np_dtype_to_tensor_dtype(
                np.dtype(ir.type.toNumpy()))
        value_info = onnx.helper.make_tensor_value_info(
            ir.name, t, ir.shape, ir.doc_string)
        return value_info

    def parse_tensor(self, ir: Variable):
        data = ir.data
        assert data is not None
        if isinstance(data, LazyData):
            tensor = data.tensor
        else:
            tensor = onnx.numpy_helper.from_array(data.getNp())
            if data.location is not None:
                tensor.data_location = data.location
        tensor.name = ir.name
        return tensor

    def parse_node(self, ir: Node):
        node = onnx.helper.make_node(
            ir.op_type,
            inputs=[v.name for v in ir.input],
            outputs=[v.name for v in ir.output],
            name=ir.name,
            domain=ir.domain,
            doc_string=ir.doc_string
        )
        for key, val in ir.attrs.items():
            if isinstance(val, Variable):
                val = self.parse_tensor(val)
            elif isinstance(val, Graph):
                val = self.parse_graph(val)
            else:
                pass
            node.attribute.extend([onnx.helper.make_attribute(key, val)])
        return node



from typing import Union
from PySide6.QtGui import QPainter, QPen, QColor, QKeyEvent, QWheelEvent, QTransform
from PySide6.QtWidgets import QGraphicsView, QGraphicsItem
from PySide6.QtCore import Qt, QRectF, QRect, QLineF, QPointF, Slot
from ..ir import Graph
from .graphics.scene import GraphScene
import math
from .ui import FindBar


class GraphEditor(QGraphicsView):
    def __init__(self, ir: Graph, parent=None):
        super().__init__(parent)
        self.viewport().setAttribute(Qt.WidgetAttribute.WA_AcceptTouchEvents, False)
        self._ir: Graph = ir

        self.setDragMode(QGraphicsView.DragMode.ScrollHandDrag)
        self.setHorizontalScrollBarPolicy(
            Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.setTransformationAnchor(
            QGraphicsView.ViewportAnchor.AnchorUnderMouse)

        s = GraphScene(self._ir, self)
        box, first_n = s.layout()
        self.setScene(s)

        if box is not None:
            box.adjust(0, -50, 0, 50)
            self.setSceneRect(box)
            self.setScale(1.5)
        if first_n is not None:
            self.centerOn(first_n)

        self.setBackgroundBrush(QColor(53, 53, 53))

        self._find_bar = FindBar(self._ir, self)
        self._find_bar.centerOn.connect(self.focusOn)

    @property
    def name(self):
        return self._ir.name

    def drawBackground(self, painter: QPainter, rect: Union[QRectF, QRect]) -> None:
        super().drawBackground(painter, rect)

        def drawGrid(gridStep):
            windowRect = self.rect()
            tl = self.mapToScene(windowRect.topLeft())
            br = self.mapToScene(windowRect.bottomRight())
            left = math.floor(tl.x() / gridStep - 0.5)
            right = math.floor(br.x() / gridStep + 1.0)
            bottom = math.floor(tl.y() / gridStep - 0.5)
            top = math.floor(br.y() / gridStep + 1.0)
            for xi in range(left, right+1):
                painter.drawLine(
                    QLineF(xi * gridStep, bottom * gridStep, xi * gridStep, top * gridStep))
            for yi in range(bottom, top+1):
                painter.drawLine(QLineF(left * gridStep, yi *
                                 gridStep, right * gridStep, yi * gridStep))

        pfine = QPen(QColor(60, 60, 60), 1.0)
        painter.setPen(pfine)
        drawGrid(15)
        pfine = QPen(QColor(25, 25, 25), 1.0)
        painter.setPen(pfine)
        drawGrid(150)

    def keyPressEvent(self, event: QKeyEvent) -> None:
        if event.key() == Qt.Key.Key_Shift:
            self.setDragMode(QGraphicsView.DragMode.RubberBandDrag)
        return super().keyPressEvent(event)

    def keyReleaseEvent(self, event: QKeyEvent) -> None:
        if event.key() == Qt.Key.Key_Shift:
            self.setDragMode(QGraphicsView.DragMode.ScrollHandDrag)
        return super().keyReleaseEvent(event)

    def wheelEvent(self, event: QWheelEvent) -> None:
        delta = event.angleDelta()
        if delta.y() == 0:
            event.ignore()
            return super().wheelEvent(event)
        d = delta.y() / abs(delta.y())
        if d > 0.0:
            self.scaleUp()
        else:
            self.scaleDown()
        # return super().wheelEvent(event)

    def scaleUp(self):
        step = 1.2
        factor = step ** 1
        t = self.transform()
        if (t.m11() > 2):
            return
        self.scale(factor, factor)

    def scaleDown(self):
        step = 1.2
        factor = step ** (-1.0)
        t = self.transform()
        if t.m11() < 0.15:
            return
        self.scale(factor, factor)

    def scaleExtremeUp(self):
        t = self.transform()
        self.scaleUp()
        while t != self.transform():
            self.scaleUp()

    def scaleExtremeDown(self):
        t = self.transform()
        self.scaleDown()
        while t != self.transform():
            self.scaleDown()

    def setScale(self, scale):
        self.setTransform(QTransform(scale, 0, 0, scale, 0, 0))

    def expand(self, f):
        rectTmp = self.scene().sceneRect()
        ptTopLeft = rectTmp.topLeft()
        ptBottomRight = rectTmp.bottomRight()
        rect = self.rect()
        ptW_H = f * QPointF(rect.width(), rect.height())
        ptTopLeft -= ptW_H
        ptBottomRight += ptW_H
        rectTmp.setTopLeft(ptTopLeft)
        rectTmp.setBottomRight(ptBottomRight)
        self.scene().setSceneRect(rectTmp)
        self.update()

    def centerTop(self):
        hBar = self.horizontalScrollBar()
        vBar = self.verticalScrollBar()
        hBar.setValue(hBar.minimum() + (hBar.maximum() - hBar.minimum()) / 2)
        vBar.setValue(vBar.minimum())
        self.update()

    @Slot(QGraphicsItem)
    def focusOn(self, obj: QGraphicsItem, clean_selected=True):
        if clean_selected:
            self.scene().clearSelection()
        self.centerOn(obj)
        obj.setSelected(True)
        self.scene().setFocusItem(obj, Qt.FocusReason.MenuBarFocusReason)

    @Slot()
    def displayFindBar(self):
        self._find_bar.show()



from onnxeditor.gui import entry
from onnxeditor.ir import Model
# from onnxeditor.ir.port import OnnxImport
import numpy as np


# imp = OnnxImport()
# m = imp('/Users/opluss/Downloads/resnet18-v1-7.onnx')
m = Model()

g = m.graph

n0t0 = g.addNode('name0', 'opt0')

n0t0.setAttr('a0', "sdfasfs")
n0t0.setAttr('sadf', 123)
n0t0.setAttr('cc', [1, 2, 3])
n0t0.setAttr('data', np.array([1, 2, 34]))

# n1t1 = g.addNode('name1', 'opt1')
# n2t1 = g.addNode('name2', 'opt1')
# n3t2 = g.addNode('name3', 'opt2')

g.markInput('v0')

c0 = g.addVariable('c0')
c0.data = np.array([1, 2, 3])

n0t0.input = ['v0']
n0t0.output = ['v1']

g.markOutput('v1')

# n1t1.input = ['c0']
# n1t1.output = ['v2']
# n2t1.input = ['v1', 'v2']
# n2t1.output = ['v3', 'v4']
# n3t2.input = ['v3', 'v2']
# n3t2.output = ['v5']


# g.markOutput('c0')
# g.markOutput('v2')
# g.markOutput('v5')

# print(m)

entry(m)



from typing import Any, Optional, TYPE_CHECKING
import PySide6.QtGui
from PySide6.QtWidgets import QGraphicsItem, QGraphicsSceneHoverEvent, QGraphicsSceneMouseEvent, QStyleOptionGraphicsItem, QWidget, QDialog
from PySide6.QtCore import Signal, Qt, QRectF, QLineF, QPointF
from PySide6.QtGui import QPainter, QColor, QPen, QFont, QFontMetrics
from ...ir import Node, Variable
import math
from .node import GraphNode
from ..ui import NodeSummary

if TYPE_CHECKING:
    from .edge import GraphEdge


class NormalGraphNode(GraphNode):

    def __init__(self, ir: Node, parent=None):
        super().__init__(parent)
        self._ir: Node = ir

        self._attr_layout = []
        self._line = QLineF()
        self._border = QRectF()

        self.layoutWith(self._ir.op_type, None, self._ir.attrs)

        def io_change(v: Node):
            assert v.read_ext('bind_gnode') == self
            self.connectToEdge()
            self.io_change.emit([self])

        self._ir.input_change_callback = io_change
        self._ir.output_change_callback = io_change

    @property
    def ir(self):
        return self._ir

    def connectToEdge(self):
        '''
        here, we just connect the signal, edge will auto disconnect the unreachable link
        '''
        for i in self._ir.input:
            e: GraphEdge = i.read_ext('bind_gedge')
            self.pos_move.connect(e.needUpdate)
            self.io_change.connect(e.needUpdate)
        for o in self._ir.output:
            e: GraphEdge = o.read_ext('bind_gedge')
            self.pos_move.connect(e.needUpdate)
            self.io_change.connect(e.needUpdate)

    def mouseDoubleClickEvent(self, event: QGraphicsSceneMouseEvent) -> None:
        dialog = NodeSummary(self._ir)
        dialog.setWindowTitle('Edit Node')
        ret = dialog.exec()
        if ret == QDialog.DialogCode.Accepted:
            ret = dialog.getRet()
            self._ir.name = ret['name']
            self._ir.op_type = ret['op_type']
            self._ir.input = ret['inputs']
            self._ir.output = ret['output']
            self._ir.clearAttr()
            for k, v in ret['attrs'].items():
                self._ir.setAttr(k, v)
            self.layoutWith(self._ir.op_type, None, self._ir.attrs)
        return super().mouseDoubleClickEvent(event)



from setuptools import setup, find_packages
from setuptools.command.build_py import build_py
import os

GEN_LIST = [
    ("onnxeditor/gui/res", "res.qrc", "res.py"),
    ("onnxeditor/gui/ui/iosummary", "iosummary.ui", "ui_iosummary.py"),
    ("onnxeditor/gui/ui/nodesummary", "nodesummary.ui", "ui_nodesummary.py"),
    ("onnxeditor/gui/ui/datainspector", "datainspector.ui", "ui_datainspector.py"),
]


class BuildQtFileCommand(build_py):
    def run(self):
        for item in GEN_LIST:
            if item[1].endswith('.qrc'):
                os.system(
                    f'cd {item[0]} && pyside6-rcc {item[1]} -o {item[2]}')
            elif item[1].endswith('.ui'):
                os.system(
                    f'cd {item[0]} && pyside6-uic {item[1]} -o {item[2]}')
            else:
                raise NotImplementedError(f'Not handled suffix: {item[1]}')
        build_py.run(self)


setup(
    name='onnxeditor',
    version='0.0.9',
    packages=find_packages(),
    install_requires=[
        'numpy',
        'pyside6',
        'grandalf',
        'onnx',
    ],
    python_requires='>=3.6',
    author='oPluss',
    author_email='opluss@qq.com',
    description='A Qt base onnx editor',
    long_description='A Qt base onnx editor',
    url='https://github.com/OYCN/OnnxEditorV3',
    cmdclass={
        'build_py': BuildQtFileCommand,
    },
    entry_points={
        'console_scripts': [
            'onnxeditor=onnxeditor:main',
        ],
    },
)



from typing import Any, Optional, TYPE_CHECKING
import PySide6.QtGui
from PySide6.QtWidgets import QGraphicsItem, QGraphicsSceneHoverEvent, QGraphicsSceneMouseEvent, QStyleOptionGraphicsItem, QWidget, QDialog
from PySide6.QtCore import Signal, Qt, QRectF, QLineF, QPointF
from PySide6.QtGui import QPainter, QColor, QPen, QFont, QFontMetrics
from ...ir import Variable
import math
from .node import GraphNode
from ..ui import IOSummary

if TYPE_CHECKING:
    from .edge import GraphEdge


class IOGraphNode(GraphNode):

    def __init__(self, ir: Variable, parent=None):
        super().__init__(parent)
        self._ir: Variable = ir

        self._title = None
        self._border = QRectF()

        self.layoutWith(self._ir.name, None, None)

    @property
    def ir(self):
        return self._ir

    def connectToEdge(self):
        e: GraphEdge = self._ir.read_ext('bind_gedge')
        self.pos_move.connect(e.needUpdate)
        self.io_change.connect(e.needUpdate)

    def mouseDoubleClickEvent(self, event: QGraphicsSceneMouseEvent) -> None:
        dialog_name = 'Input' if self._ir.isInput else 'Output'
        assert self._ir.graph is not None
        dialog = IOSummary(self._ir, self._ir.graph.variables)
        dialog.setWindowTitle('Edit ' + dialog_name)
        ret = dialog.exec()
        if ret == QDialog.DialogCode.Accepted:
            ret = dialog.getRet()
            assert self._ir.name == ret['name']
            self._ir.shape = ret['shape']
            self._ir.type = ret['type']
            self.layoutWith(self._ir.name, None, None)
        return super().mouseDoubleClickEvent(event)



from PySide6.QtGui import QKeyEvent
from PySide6.QtWidgets import QGraphicsScene, QMenu, QGraphicsSceneContextMenuEvent, QDialog, QMessageBox
from PySide6.QtCore import Signal, Slot, Qt, QRectF, QPointF
from ...ir import Graph, Node, Variable
from .node import GraphNode
from .normal_node import NormalGraphNode
from .io_node import IOGraphNode
from .edge import GraphEdge
from typing import List, Union
from grandalf.graphs import graph_core as GC
from grandalf.graphs import Graph as GG
from grandalf.graphs import Vertex as NN
from grandalf.graphs import Edge as EE
from grandalf.layouts import SugiyamaLayout
from ..ui import IOSummary, NodeSummary, DataInspector
import time
from typing import Union


class GraphScene(QGraphicsScene):
    def __init__(self, ir: Graph, parent=None):
        super().__init__(parent)
        self._ir: Graph = ir
        self._normal_node = []
        self._io_node = []
        self._edge = []
        if self._ir is not None:
            def del_nodeitem(o: Node):
                assert o.read_ext('bind_gnode') is not None
                n: GraphNode = o.read_ext('bind_gnode')
                n.setPos(0, 0)
                self.removeItem(n)

            def del_io_item(input: bool):
                def f(o: Variable):
                    n = o.read_ext(
                        'bind_gnode_src' if input else 'bind_gnode_dst')
                    n: GraphNode = o.read_ext('bind_gnode')
                    n.setPos(0, 0)
                    self.removeItem(n)
                return f

            def add_io_item(input: bool):
                def f(o: Variable):
                    self.bind_node(o, input)
                return f
            self._ir.var_add_callback = self.bind_edge
            self._ir.node_add_callback = self.bind_node
            self._ir.node_del_callback = del_nodeitem
            self._ir.var_mark_input_callback = add_io_item(True)
            self._ir.var_mark_output_callback = add_io_item(False)
            self._ir.var_unmark_input_callback = del_io_item(True)
            self._ir.var_unmark_output_callback = del_io_item(False)
            # init var before node
            # because signal connect in node
            for v in self._ir.variables:
                self.bind_edge(v)
            for v in self._ir.input:
                self.bind_node(v, True)
            for v in self._ir.output:
                self.bind_node(v, False)
            for n in self._ir.nodes:
                self.bind_node(n)

    def bind_node(self, ir: Union[Node, Variable], asinput: bool = False):
        if isinstance(ir, Node):
            if ir.op_type in ['Constant']:
                # we will skip display constant
                return None
            n = NormalGraphNode(ir, self)
            self._normal_node.append(n)
            assert ir.read_ext('bind_gnode') is None
            ir.set_ext('bind_gnode', n)
        else:
            assert isinstance(ir, Variable)
            n = IOGraphNode(ir, self)
            self._io_node.append(n)
            ir.set_ext('bind_gnode_last', n)
            if asinput:
                assert ir.read_ext('bind_gnode_src') is None
                ir.set_ext('bind_gnode_src', n)
            else:
                assert ir.read_ext('bind_gnode_dst') is None
                ir.set_ext('bind_gnode_dst', n)
        self.addItem(n)
        n.connectToEdge()
        return n

    def bind_edge(self, ire: Variable):
        e = GraphEdge(ire, self)
        self._edge.append(e)
        assert ire.read_ext('bind_gedge') is None
        ire.set_ext('bind_gedge', e)
        self.addItem(e)
        return e

    def layout(self):
        ts = time.time()
        print('cvt to layout ir')
        N = {n.id: NN(n) for n in self._normal_node}
        N.update({n.id: NN(n) for n in self._io_node})
        if len(N) == 0:
            print('skip layout because empty graph')
            return (None, None)
        E = {}
        for n in N.values():
            n = n.data
            if isinstance(n, NormalGraphNode):
                src = [nn.read_ext('bind_gnode').id for nn in n.ir.prevNodes]
                dst = [nn.read_ext('bind_gnode').id for nn in n.ir.nextNodes]
            elif isinstance(n, IOGraphNode):
                src = [nn.read_ext('bind_gnode').id for nn in n.ir.src]
                dst = [nn.read_ext('bind_gnode').id for nn in n.ir.dst]
                if n.ir.read_ext('bind_gnode_src') is not None:
                    src += [n.ir.read_ext('bind_gnode_src').id]
                if n.ir.read_ext('bind_gnode_dst') is not None:
                    dst += [n.ir.read_ext('bind_gnode_dst').id]
            else:
                raise RuntimeError(f'Unexcept type: {type(n)}')
            for s in src:
                if s in N:
                    E[(n.id, s)] = EE(N[n.id], N[s])
            for d in dst:
                if d in N:
                    E[(d, n.id)] = EE(N[d], N[n.id])
        print('cvt done:', time.time() - ts, 's')
        ts = time.time()
        print('N: ', len(N))
        print('E: ', len(E))
        print('gen gc')
        g = GG(list(N.values()), list(E.values()))
        print('gen done:', time.time() - ts, 's')
        print('graph_core num:', len(g.C))
        print('graph e num:', len(list(g.E())))
        print('graph v num:', len(list(g.V())))
        ts = time.time()
        print('layout start')
        for n in N.values():
            rect = n.data.boundingRect()
            assert rect is not None

            class HWView(object):
                w, h = rect.width(), rect.height()
            n.view = HWView()
            # print(n.data.ir.name, n.view.w, n.view.h)
        sug = SugiyamaLayout(g.C[0])
        sug.init_all()
        sug.draw()
        print('layout done:', time.time() - ts, 's')
        ts = time.time()
        print('layout appply')
        box = QRectF()
        top_node = None
        for n in g.C[0].sV:
            x, y = n.view.xy
            w = n.view.w
            h = n.view.h
            x = x - w / 2
            y = - y - h / 2
            this_box = QRectF(x, y, w, h)
            box |= this_box
            # print(n.data.ir.name, x, y)
            n.data.setPos(x, y)
            # print(n.data.ir.name, n.data.pos())
            if top_node is None:
                top_node = n.data
            else:
                if n.data.pos().y() < top_node.pos().y():
                    top_node = n.data
        print('apply done:', time.time() - ts, 's')
        top_input_node = None
        for n in self._io_node:
            if top_input_node is None:
                top_input_node = n
            else:
                if n.pos().y() < top_input_node.pos().y():
                    top_input_node = n
        first_node = top_node if top_input_node is None else top_input_node
        print(f'all done, box={box}, first_node: {first_node.pos()}')
        return (box, first_node)

    def contextMenuEvent(self, event: QGraphicsSceneContextMenuEvent) -> None:
        super().contextMenuEvent(event)

        def add_node():
            n = self.addNodeDialog()
            if n is not None:
                n.setPos(event.scenePos())

        def add_io(input):
            def f():
                n = self.addIODialog(input)
                if n is not None:
                    n.setPos(event.scenePos())
            return f
        m = QMenu()
        new_node_act = m.addAction("New Node")
        new_node_act.triggered.connect(add_node)
        new_input_act = m.addAction("New Input")
        new_input_act.triggered.connect(add_io(True))
        new_output_act = m.addAction("New Output")
        new_output_act.triggered.connect(add_io(False))
        add_constant_act = m.addAction("Add Constant")
        add_constant_act.triggered.connect(self.addConstantDialog)
        m.exec(event.screenPos())
        event.accept()

    def addNode(self, name, op_type, inputs, outputs, attrs):
        n = self._ir.addNode(name, op_type)
        n.input = inputs
        n.output = outputs
        n.clearAttr()
        for k, v in attrs.items():
            n.setAttr(k, v)
        assert n.read_ext('bind_gnode') is not None
        return n.read_ext('bind_gnode')

    def addIO(self, name, shape, type, input):
        v = self._ir.getVariable(name)
        if input:
            if v.isInput:
                QMessageBox.warning(
                    self.parent(), 'Add Failed', f'This input already exist: {v.name}')
                return
            self._ir.markInput(v)
        else:
            if v.isOutput:
                QMessageBox.warning(
                    self.parent(), 'Add Failed', f'This output already exist: {v.name}')
                return
            self._ir.markOutput(v)
        v.shape = shape
        v.type = type
        assert v.read_ext('bind_gedge') is not None
        n = v.read_ext('bind_gnode_last')
        v.set_ext('bind_gnode_last', None)
        assert n is not None
        return n

    def delNode(self, n: NormalGraphNode):
        assert isinstance(n, NormalGraphNode)
        n.ir.removeFromGraph()

    def delIO(self, n: IOGraphNode):
        assert isinstance(n, IOGraphNode)
        if n.ir.isInput:
            n.ir.unMarkInput()
        if n.ir.isOutput:
            n.ir.unMarkOutput()

    def addNodeDialog(self):
        dialog = NodeSummary(g=self._ir)
        dialog.setWindowTitle('Add Node')
        ret = dialog.exec()
        if ret == QDialog.DialogCode.Accepted:
            ret = dialog.getRet()
            return self.addNode(ret['name'], ret['op_type'], ret['inputs'], ret['output'], ret['attrs'])
        return None

    def addIODialog(self, input=True):
        dialog_name = 'Input' if input else 'Output'
        dialog = IOSummary(None, [
                           v for v in self._ir.variables if v.isInput == (not input) and v.isOutput == input])
        dialog.setWindowTitle('Add ' + dialog_name)
        ret = dialog.exec()
        if ret == QDialog.DialogCode.Accepted:
            ret = dialog.getRet()
            return self.addIO(ret['name'], ret['shape'], ret['type'], input)
        return None

    def addConstantDialog(self):
        dialog = DataInspector()
        dialog.setWindowTitle('Add Constant')
        ret = dialog.exec()
        if ret == QDialog.DialogCode.Accepted:
            data, name = dialog.getRet()
            self._ir.getVariable(name).data = data

    def keyPressEvent(self, event: QKeyEvent) -> None:
        super().keyPressEvent(event)
        if event.isAccepted():
            return
        if event.key() in [Qt.Key.Key_Delete, Qt.Key.Key_Backspace]:
            for item in self.selectedItems():
                if isinstance(item, NormalGraphNode):
                    self.delNode(item)
                elif isinstance(item, IOGraphNode):
                    self.delIO(item)



from typing import Optional, Union, List
import PySide6.QtCore
from PySide6.QtWidgets import QDialog, QWidget, QDialogButtonBox, QCompleter
from PySide6.QtCore import Qt, Slot, Signal
from ....ir import Variable, TensorType
from .ui_iosummary import Ui_IOSummary
import re


class IOSummary(QDialog):
    def __init__(self, vir: Union[Variable, None] = None, vars: Union[List[Variable], None] = None, parent: Union[QWidget, None] = None) -> None:
        super().__init__(parent)
        self._ui = Ui_IOSummary()
        self._ui.setupUi(self)

        self._ui.type_edit.addItems([e.value for e in TensorType])

        if vir is not None:
            self._ui.name_edit.setText(vir.name)
            self._ui.name_edit.setDisabled(True)
            self._ui.dim_edit.setText(','.join([str(v) for v in vir.shape]))
            idx = self._ui.type_edit.findText(vir.type.value)
            self._ui.type_edit.setCurrentIndex(idx)
        self._vars = vars
        if vars is not None:
            cmp = QCompleter([v.name for v in vars], self)
            self._ui.name_edit.setCompleter(cmp)
        self._ui.name_edit.textChanged.connect(self.dimAutoFill)
        self._ui.dim_edit.textChanged.connect(self.dimCheckSlot)
        self.dimCheckSlot()

        self._ui.dim_auto.setChecked(True)
        self._ui.type_auto.setChecked(True)

    def checkDim(self):
        txt = self._ui.dim_edit.text()
        return re.fullmatch(r'^((-?[0-9a-zA-Z]+),?)*$', txt) is not None

    def parseDim(self):
        txt = self._ui.dim_edit.text()
        assert self.checkDim()
        ret = re.findall(r'(-?[0-9a-zA-Z]+)', txt)
        return list(ret)

    def getRet(self):
        return {
            'name': self._ui.name_edit.text(),
            'shape': self.parseDim(),
            'type': TensorType(self._ui.type_edit.currentText()),
        }

    @Slot()
    def dimCheckSlot(self):
        b = self._ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok)
        b.setEnabled(self.checkDim())

    @Slot()
    def dimAutoFill(self):
        if self._vars is None:
            return
        name = self._ui.name_edit.text()
        for v in self._vars:
            if v.name == name:
                if self._ui.dim_auto.isChecked():
                    self._ui.dim_edit.setText(
                        ','.join([str(v) for v in v.shape]))
                if self._ui.type_auto.isChecked():
                    idx = self._ui.type_edit.findText(v.type.value)
                    self._ui.type_edit.setCurrentIndex(idx)



from typing import TYPE_CHECKING, List, Dict, Any
from .obj import IRObj
import json
import numpy as np
import copy

if TYPE_CHECKING:
    from .graph import Graph
    from .variable import Variable


class Node(IRObj):
    def __init__(self, graph: 'Graph', name: str, op_type: str) -> None:
        super().__init__()

        self.name: str = name
        self.op_type: str = op_type
        self.domain: str = None
        self.doc_string: str = None
        self._attrs: Dict[str: Any] = {}
        self._input: List[str] = []
        self._output: List[str] = []

        self._graph: 'Graph' = graph

        self.displayAttr('name')
        self.displayAttr('op_type')
        self.displayAttr('domain')
        self.displayAttr('doc_string')
        self.displayAttr('attrs')
        self.displayAttr('_input', name='input')
        self.displayAttr('_output', name='output')

        self.input_change_callback = None
        self.output_change_callback = None
        self.node_remove_callback = None

        self._ext['bind_gnode'] = None

    @property
    def input(self):
        if self._graph is None:
            return []
        return [self._graph.getVariable(v) for v in self._input]

    @input.setter
    def input(self, new_ins):
        if self._graph is None:
            raise RuntimeError(
                f'Can not set input for no graph node, using self._input for setting/reading by str')
        new_ins = [self._graph.getVariable(v) if isinstance(
            v, str) else v for v in new_ins]
        old_ins = [self._graph.getVariable(v) for v in self._input]
        self._input = [v.name for v in new_ins]
        old_ins = set([v.name for v in old_ins])
        new_ins = set([v.name for v in new_ins])
        diff_old_ins = old_ins - new_ins
        diff_new_ins = new_ins - old_ins
        if len(diff_old_ins) + len(diff_new_ins) > 0:
            self._graph._emit_node_input_change(
                self, diff_old_ins, diff_new_ins)
            if self.input_change_callback is not None:
                self.input_change_callback(self)

    @property
    def output(self):
        if self._graph is None:
            return []
        return [self._graph.getVariable(v) for v in self._output]

    @output.setter
    def output(self, new_outs):
        if self._graph is None:
            raise RuntimeError(
                f'Can not set output for no graph node, using self._output for setting/reading by str')
        new_outs = [self._graph.getVariable(v) if isinstance(
            v, str) else v for v in new_outs]
        old_outs = [self._graph.getVariable(v) for v in self._output]
        self._output = [v.name for v in new_outs]
        old_outs = set([v.name for v in old_outs])
        new_outs = set([v.name for v in new_outs])
        diff_old_outs = old_outs - new_outs
        diff_new_outs = new_outs - old_outs
        if len(diff_old_outs) + len(diff_new_outs) > 0:
            self._graph._emit_node_output_change(
                self, diff_old_outs, diff_new_outs)
            if self.output_change_callback is not None:
                self.output_change_callback(self)

    @property
    def attrs(self):
        return self._attrs

    def clearAttr(self):
        self._attrs.clear()

    def setAttr(self, key, val):
        if isinstance(val, np.ndarray):
            from .variable import Variable
            val = Variable(None, '', data=val)
        self._attrs[key] = val

    @property
    def prevNodes(self):
        return [n for v in self.input for n in v._src_nodes]

    @property
    def nextNodes(self):
        return [n for v in self.output for n in v._dst_nodes]

    @property
    def graph(self):
        return self._graph

    @graph.setter
    def graph(self, g):
        assert self._graph is None
        self._graph = g
        self._graph._emit_node_output_change(
            self, [], self._output)
        self._graph._emit_node_input_change(
            self, [], self._input)

    def removeFromGraph(self):
        assert self._graph is not None
        self._graph._emit_node_output_change(
            self, self._output, [])
        self._graph._emit_node_input_change(
            self, self._input, [])
        self._graph._nodes.remove(self)
        callback = self._graph.node_del_callback
        self._graph = None
        if self.node_remove_callback is not None:
            callback = self.node_remove_callback
        if callback is not None:
            callback(self)

    def copyOut(self):
        new = copy.copy(self)
        new._graph = None
        return copy.deepcopy(new)



import os
# import sys

try:
    from .res import *
except ImportError:
    dir = os.path.dirname(__file__)
    # sys.path.append(dir)
    target = os.path.join(dir, 'res.py')
    if not os.path.exists(target):
        os.system(f'cd {dir} && pyside6-rcc res.qrc -o res.py')

from .res import *



from PySide6.QtWidgets import QMainWindow, QTabWidget, QMenu, QFileDialog, QMessageBox
from PySide6.QtGui import QIcon, QAction, QKeySequence
from PySide6.QtCore import Slot, Qt
from .graph_editor import GraphEditor
from ..ir import Model, OnnxImport, OnnxExport, pass_const_to_var
from .ui import ModelEditor
import os
import onnx
import onnx.checker
from typing import Union


class MainWindow(QMainWindow):
    def __init__(self, irm: Union[Model, None] = None, path: Union[str, None] = None, parent=None):
        super().__init__(parent)
        self._imp = OnnxImport(pass_const_to_var)
        self._exp = OnnxExport()

        self.setWindowIcon(QIcon(":/img/appicon.ico"))
        self.resize(800, 600)
        # we will connect some signal to loaded ge
        self._lk2ge = []
        self.initActions()

        self._path = ''
        self._irm: Union[Model, None] = None
        self._ge: Union[GraphEditor, None] = None
        if irm is not None or path is not None:
            self.openFile(irm, path)

    def initActions(self):
        def addMenu(name: str, menu: QMenu = None):
            if menu is None:
                return self.menuBar().addMenu(name)
            else:
                return menu.addMenu(name)

        def addAction(menu: QMenu, name: str, fn=None):
            act = QAction(self)
            act.setText(name)
            if fn is not None:
                act.triggered.connect(fn)
            menu.addAction(act)
            return act
        # File
        menu = addMenu('File')
        act = addAction(menu, "Open File", self.fileOpenSlot)
        act.setStatusTip("Open an exist onnx file")
        act.setShortcut(QKeySequence('Ctrl+o'))
        act = addAction(menu, "Save", self.fileSaveSlot)
        act.setStatusTip("Save this onnx file")
        act.setShortcut(QKeySequence('Ctrl+s'))
        act = addAction(menu, "Save as", self.fileSaveAsSlot)
        act.setStatusTip("Save this onnx file as new file")
        act.setShortcut(QKeySequence('Ctrl+e'))
        # Edit
        menu = addMenu('Edit')
        act = addAction(menu, "Find")
        self._lk2ge.append(
            lambda ge, act=act: act.triggered.connect(ge.displayFindBar))
        act.setStatusTip("Display Find Bar")
        act.setShortcut(QKeySequence('Ctrl+f'))
        act = addAction(menu, "Model Properties", self.showModelEditDialog)
        act.setStatusTip("Edit model properties")

    def openFile(self, irm: Union[Model, None], path: Union[str, None]):
        if path is None:
            path = ''
            self._path = path
        elif irm is None:
            m = onnx.load(path)
            try:
                onnx.checker.check_model(m)
            except Exception as e:
                msb = QMessageBox(QMessageBox.Icon.Warning,
                                  "onnx checker error", str(e), parent=self)
                msb.setWindowModality(Qt.WindowModality.WindowModal)
                msb.show()
            irm = self._imp(path)
        if not path.startswith('(') and len(path) > 0:
            path = '(' + path + ')'
        self.setWindowTitle('OnnxEditor' + path)
        if irm is None:
            irm = Model()
        self._irm = irm
        self._ge = GraphEditor(irm.graph)
        self.setCentralWidget(self._ge)
        for fn in self._lk2ge:
            fn(self._ge)

    @Slot()
    def fileOpenSlot(self):
        path = QFileDialog.getOpenFileName(
            self, "open onnx file", "/", '*.onnx')
        if path is None or len(path[0]) == 0:
            return
        else:
            self.openFile(None, path[0])

    @Slot()
    def fileSaveSlot(self):
        if len(self._path) == 0:
            self.fileSaveAsSlot()
        else:
            m = self._exp(self._irm, self._path)
            try:
                onnx.checker.check_model(m)
            except Exception as e:
                QMessageBox.warning(self, "onnx checker error", str(e))

    @Slot()
    def fileSaveAsSlot(self):
        if len(self._path) == 0:
            self._path = "/"
        else:
            self._path = os.path.dirname(self._path)
        path = QFileDialog.getSaveFileName(
            self, "save onnx file", self._path, '*.onnx')
        if path is None or len(path[0]) == 0:
            return
        else:
            m = self._exp(self._irm, path[0])
            try:
                onnx.checker.check_model(m)
            except Exception as e:
                QMessageBox.warning(self, "onnx checker error", str(e))

    @Slot()
    def showModelEditDialog(self):
        dialog = ModelEditor(self._irm, self)
        dialog.exec()
        ret = dialog.getRet()
        for k, v in ret.items():
            setattr(self._irm, k, v)
